/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ResourceLink");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.DomParser");

dojo.require("jazz.ui.internal.ResourceLinkAggregator");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.util.html");
dojo.require("jazz.util.internal.login");
dojo.require("jazz.client.xhr");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui.internal", "ResourceLinkMessages");

(function(){

var CSS_RL = "jazz-ui-ResourceLink";
var CSS_ARROW = "arrow";
var CT_COMPACT_RENDERING = "application/x-jazz-compact-rendering";
var PREVIEW_RESIZE_MESSAGE_PREFIX = "jazz-compact-rendering-preview-height:";
var XMLNS_DC_TERMS = "http://purl.org/dc/terms/";
var XMLNS_JP = "http://jazz.net/xmlns/prod/jazz/presentation/1.0/";
var XMLNS_RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

var defaultLazyFetch = false;
var messages = null;

dojo.declare("jazz.ui.ResourceLink", dijit._Widget, {
    
	// summary:
	//		A hyperlink referencing a Jazz resource. We take the URL of the
	//		link's HREF attribute to determine the resource (Work Item, Plan,
	//		etc.) and then send back information to display a summary of the
	//		resource when the user hovers over the link. The summaries presented
	//		can have a small and large view depending on how much content the
	//		user wishes to see.
	//
	// parameters:
	//		uri: (String) The uri that will be used to find the available
	//				presentations. The URI can be relative ("/testURI") 
	//				or absolute ("http://localhost/testURI"). REQUIRED if 
	//				the given linkNode does not have a uri attached to 
	//				the HREF attribute.
	//		(optional) presentationUri: (String) A URL that will replace the
	//				link's HREF attribute, used to send a user to a different
	//				presentation of the given resource.
	//		(optional) label: (String) A string or HTML snippet that will be
	//				placed as the child node/inner text of the link/anchor node
	//				the Resource Link will be attached to.  If no srcNodeRef is 
	//				given, this argument is REQUIRED.
	//		(optional) useAbbreviation: (boolean) If set to true, the text of the
	//				anchor node will be replaced with the compact rendering's
	//				"jp:abbreviation" value. If none, exists then the widget will
	//				look at the value of the retainLinkText. If it is false, the
	//				widget will fall back to the full resource title and if it is
	//				true, it will leave the link text unchanged. By default, this
	//				false.
	//		(optional) retainLinkText: (boolean) If set to false, the text in the 
	//				srcNodeRef anchor node will change to the text of the compact 
	//				rendering's "dc:title" attribute. If set to true, the text of 
	//				the srcNodeRef anchor node will not change when the resource 
	//				link is created. By default, this parameter is set to false 
	//				and will change the text of the anchor node. Note, this
	//				parameter is ignored if useAbbreviation is true and the compact
	//				rendering document supplies a "jp:abbreviation".
	//		(optional) iconNode: (DOM Node reference or id) The DOM node in the page
	//				to that should be used to render the icon. If an <img> node is
	//				provided, the src attribute will be update on it directly. If any
	//				other node is provided, an <img> tag will be created and appended
	//				to it. This parameter can also be the string id of the desired
	//				node.
	//		(optional) lazyFetch: (boolean) If set to true, the Resource Link will
	//				will not fetch the compact rendering document until the user
	//				hovers over the link.  Also, if this option is set to true,
	//				the "retainLinkText" parameter will automatically be set to true,
	//				and the "iconNode" parameter will be ignored.  By default,
	//				this parameter is set to false, and the compact rendering document
	//				will be aggressively fetched when the page loads.
	//		(optional) srcNodeRef: (Anchor Node) The anchor node in the page
	//				that the Resource Link will be attached to. NOTE: if the uri
	//				parameter is not given, the href value of the given	node
	//				will be used in its place as the resource to reference.
	//
	// methods:
	//
	//		refresh: if the resource that the Resource Link is viewing has changed,
	//				one can call this method to retrieve the new compact rendering
	//				document and update the recource link hover with new data.
	//
	// usage:
	//		var navLink = new jazz.ui.ResourceLink({}, linkNode);
	//		-or-
    //		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//                  label: "Label for the Anchor Node"
	//				});
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//				},
	//				srcNodeRef);
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl"
	//				},
	//				srcNodeRef);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "New Label for the Anchor Node"
	//				},
	//				srcNodeRef);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "Label for the Anchor Node"
	//				});
	// since:
	//		0.7
	
	presentationUri: "",
	useAbbreviation: false,
	retainLinkText: false,
	iconNode: "",
	lazyFetch: false,
	
	buildRendering: function(){
		if (!this.srcNodeRef && this.uri) {
			this.domNode = document.createElement("A");
			if (!this.label)
				throw new Error("ResourceLink REQUIRES a label parameter if a uri is given rather than a reference to an Anchor Node.")
		}
		else if (this.srcNodeRef && this.srcNodeRef.tagName === "A") {
			this.domNode = this.srcNodeRef;
			if (!this.uri)
				this.uri = this.domNode.href;
		}
		else
			throw new Error("ResourceLink REQUIRES that either the uri parameter or a valid reference to an Anchor Node is given");
		
		if (!this.uri.match(/http(?:s)?\:\/\//)) {
			var baseUri = document.baseURI || document.URL; //document.URL is used to acquire the base URI in IE
			this.uri = new dojo._Url(baseUri, this.uri).toString();
		}
		this.domNode.href = this.presentationUri || this.uri;
		if (this.label)
			this.domNode.innerHTML = jazz.util.html.removeScript(this.label);
		//this._styleLink();
	},
	
	postCreate: function() {
		messages = dojo.i18n.getLocalization("jazz.ui.internal", "ResourceLinkMessages");
		
		if (this.iconNode)
			this.iconNode = dojo.byId(this.iconNode);
		if (this.lazyFetch || (defaultLazyFetch && this.lazyFetch !== false))
			this.lazyFetch = true;
		else
			this._registerWithAggregator(this._batchedPresentationHandler, this._errorPresentation);

		this.popup = new net.jazz.ui._RichHover({
			around: this.domNode,
			uri: this.uri,
			presentationUri: this.presentationUri,
			lazyFetch: this.lazyFetch,
			onDisable: dojo.hitch(this,this._disable)
		});
	},
	
	refresh: function() {
		this._registerWithAggregator(this._batchedPresentationHandler, this._errorRefresh);
	},
	
	_registerWithAggregator: function(callback, errback) {
		jazz.ui.internal.resourceLinkAggreagtor().register(this.uri, dojo.hitch(this, callback), dojo.hitch(this, errback));
	},
	
	_styleLink: function () {
		dojo.addClass(this.domNode, CSS_RL);
		this.arrowNode = document.createElement("img");
		dojo.addClass(this.arrowNode, CSS_ARROW);
		this.arrowNode.src = dojo.moduleUrl("jazz.ui", "graphics/ResourceLink/rh-twistie.gif");
		this.domNode.appendChild(this.arrowNode);
	},
	
	_errorPresentation: function (err) {
		console.warn("ResourceLink Error (" + this.uri + "): " + err.message);
		this.popup.disable();
	},
	
	_errorRefresh: function (err) {
		console.warn("ResourceLink Refresh Error (" + this.uri + "): " + err.message);
	},
	
	_disable: function () {
		if (this.arrowNode && this.arrowNode.parentNode == this.domNode) {
			this.domNode.removeChild(this.arrowNode);
			delete this.arrowNode;
		}
		dojo.removeClass(this.domNode, CSS_RL);
		delete this.popup;
	},
	
	_batchedPresentationHandler: function(result) {
		processCompactRendering(result);
		if (result.abbreviation && this.useAbbreviation) {
			this._applyTitle(result.abbreviation);
		} else if (result.title && !this.retainLinkText) {
			this._applyTitle(result.title);
		}
		if (result.icon && this.iconNode) {
			var iconImg;
			if (this.iconNode.tagName === "IMG") {
				iconImg = this.iconNode;
			} else {
				iconImg = document.createElement("img");
				this.iconNode.appendChild(iconImg);
			}
			iconImg.src = result.icon;
		}
		
		this.popup.setCompactRendering(result);
	},
	
	_applyTitle: function(title) {
		if (this.arrowNode && this.arrowNode.parentNode)
			this.arrowNode.parentNode.removeChild(this.arrowNode);
		this.domNode.innerHTML = title;
		if (this.arrowNode)
			this.domNode.appendChild(this.arrowNode);
	},
	
	destroy: function() {
		if (this.popup)
			this.popup.destroy();
		this.inherited(arguments);
	}
	
});
jazz.ui.ResourceLink.setLazyFetch = function(bool) {
	defaultLazyFetch = bool;
};

dojo.declare("net.jazz.ui._RichHover", jazz.ui.HoverPopup, {
	customClass: "jazz-ui-ResourceHover",
	
	createContents: function() {
		if (!this.compactRendering) {
			if (this.lazyFetch) {
				this._fetchCompactRendering();
			}
			return dojo.create("div", {innerHTML: messages.loading});
		} else {
			return this._createPresentation(dojo.create("div"));
		}
	},
	
	close: function(e) {
		if (e)
			dojo.stopEvent(e);
		this.inherited(arguments);
	},
	
	onOpen: function() {
		this._openListeners = [];
		this._openListeners.push(this.connect(dojo.body(), "onkeypress", this._onKeyPress));
		this._openListeners.push(this.connect(dojo.body(), "onclick", this._onBodyClick));
		this._openListeners.push(this.connect(dojo, "stopEvent", this._onStopEvent));
		if (window.postMessage)
			this._openListeners.push(this.connect(window, "onmessage", this._onMessage));
		this._renderResult();
	},
	
	onClose: function() {
		this.programmatic = false;
		this.updateToggleLink(true);
		for (var i = 0; i < this._openListeners.length; i++)
			this.disconnect(this._openListeners[i]);
		if (this._disableOnClose)
			this.disable();
	},
	
	toggleSize: function(e) {
		dojo.stopEvent(e);
		this.programmatic = "block";
		this.updateToggleLink(this._isFullDisplay);
		if (this._isFullDisplay) {
			this.displaySmallSummary();
		} else {
			this.displayLargeSummary();
		}
		this.position();
	},
	
	updateToggleLink: function(showMore) {
		if (this._presentation) {
			dojo.addClass(this._presentation.toggleLink, showMore ? "showMore" : "showLess");
			dojo.removeClass(this._presentation.toggleLink, showMore ? "showLess" : "showMore");
			this._presentation.toggleLink.innerHTML = showMore ? messages.showMore : messages.showLess;
		}
	},
	
	displaySmallSummary: function(){
		this._isFullDisplay = false;
		if (this.compactRendering.smallPreview) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this._updateIFrame();
			this._presentation.hoverIframe.src = this.compactRendering.smallPreview;
		}
	},
	
	displayLargeSummary: function(){
		this._isFullDisplay = true;
		if (this.compactRendering.largePreview) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this._updateIFrame();
			this._presentation.hoverIframe.src = this.compactRendering.largePreview;
		}
	},
	
	disable: function(message) {
		if (this._isOpen) {
			this._renderError(message);
			this._disableOnClose = true;
			return;
		}
		if (this.onDisable)
			this.onDisable();
		this.destroy();
	},
	
	_renderResult: function() {
		if (this.compactRendering) {
			if (this.compactRendering.oauthRequired) {
				if (this._presentation) {
					this._presentation.destroy();
					delete this._presentation;
				}
				this.content.innerHTML = "";
				var loginLink = jazz.util.internal.login.insertLoginLink(this.content, this.compactRendering.token);
				this.connect(loginLink, "onclick", this.oauthLogin);
			} else {
				this._createPresentation();
				if (this.compactRendering.smallPreview) 
					this.displaySmallSummary();
				else 
					this.displayLargeSummary();
			}
		}
	},
	
	_renderError: function(message) {
		if (this._presentation) {
			this._presentation.destroy();
			delete this._presentation;
		}
		this.content.innerHTML = message;
	},
	
	setCompactRendering: function(cr) {
		this.compactRendering = cr;
		if (!cr.oauthRequired && !cr.showHover) 
			this.disable(messages.infoNotAvailable);
		else {
			if (window.postMessage) {
				if (cr.initialSmallHeight && !this.dynamicSmallHeight)
					this.dynamicSmallHeight = cr.initialSmallHeight;
				if (cr.initialLargeHeight && !this.dynamicLargeHeight && !this.dynamicSmallHeight)
					this.dynamicLargeHeight = cr.initialLargeHeight;
			}
			this._updatePresentation();
			if (this._isOpen) 
				this._renderResult();
		}
	},
	
	_updatePresentation: function() {
		var cr = this.compactRendering, pres = this._presentation;
		if (pres) {
			if (cr.title) {
				pres.titleLink.innerHTML = cr.title;
			}
			if (cr.icon) {
				pres.iconNode.src = cr.icon;
				dojo.removeClass(pres.iconLink, "hidden");
				dojo.addClass(pres.titleWrapper, "titleIndent");
			} else {
				pres.iconNode.src = "";
				dojo.addClass(pres.iconLink, "hidden");
				dojo.removeClass(pres.titleWrapper, "titleIndent");
			}
			if (cr.showExpand) {
				dojo.removeClass(pres.bottomSection, "hidden");
			} else {
				dojo.addClass(pres.bottomSection, "hidden");
			}
		}
	},
	
	_createPresentation: function(node) {
		node = node || this.content;
		if (!this.compactRendering.oauthRequired && !this._presentation) {
			this._presentation = new net.jazz.ui._RichHoverPresentation({
				close: dojo.hitch(this,this.close),
				toggleSize: dojo.hitch(this,this.toggleSize)
			});
			this._updatePresentation();
			this._presentation.titleLink.href = this._presentation.iconLink.href = this.presentationUri || this.uri;
			node.innerHTML = "";
			node.appendChild(this._presentation.domNode);
			this._presentation.hoverIframe.title = this.compactRendering.title;
		}
		return node;
	},
	
	_fetchCompactRendering: function() {
		var dfd = jazz.client.xhrGet({
			url: this.uri,
			headers: {
				accept: CT_COMPACT_RENDERING
			},
			skipInterceptors: ["jazz.oauth"]
		});
		dfd.addCallback(dojo.hitch(this, this._handleLazyFetchResponse, dfd));
		dfd.addErrback(dojo.hitch(this, function (err) {
			if (err && err.status === 401) {
				var token = new dojo._Url(this.uri).authority;
				this.setCompactRendering({
					oauthRequired: true,
					token: token
				});
				jazz.ui.internal.resourceLinkAggreagtor().registerOauth(token, this.uri, dojo.hitch(this,this._handleLazyFetchJson), dojo.hitch(this,this.disable,messages.infoNotAvailable));
			} else {
				this.disable(messages.infoNotAvailable);
			}
		}));
	},
	
	_handleLazyFetchResponse: function(dfd, response) {
		if (dfd.ioArgs.xhr.getResponseHeader("Content-Type").substring(0,CT_COMPACT_RENDERING.length) === CT_COMPACT_RENDERING) {
			var crObj = parseCompactRenderingXML(dojox.xml.DomParser.parse(response));
			this._handleLazyFetchJson(crObj);
		} else {
			this.disable(messages.infoNotAvailable);
		}
	},
	
	_handleLazyFetchJson: function(response) {
		processCompactRendering(response);
		this.setCompactRendering(response);
	},
	
	/**
	 * Access the compact rendering document on an outside server by logging in with
	 * Oauth.
	 * @param {Object} evt - mouse event
	 */
	oauthLogin: function(evt){
		dojo.stopEvent(evt);
		// handleAs "xml" doesn't work if the content-type isn't "text/xml"
		// In this case, the content type should be CT_COMPACT_RENDERING
		// so we handleAs "text" and parse the XML document in the handler
		var dfd = jazz.client.xhrGet({
			url: this.uri,
			headers: {
				"Accept": CT_COMPACT_RENDERING
			},
			handleAs: "text"
		});
		dfd.addCallback(dojo.hitch(this, this.renderOauthResponse));
		dfd.addErrback(dojo.hitch(this, this.renderOauthError));
		if (this.lazyFetch) {
			dfd.addCallback(dojo.hitch(this,this._handleLazyFetchResponse, dfd));
		}
	},
	
	/**
	 * After we authenticate with OAuth, this method has the ResourceLinkAggregator resend
	 * an xhr request for all ResourceLinks that access the same server.
	 * @param {Object} results
	 */
	renderOauthResponse: function(results) {
		var aggregator = jazz.ui.internal.resourceLinkAggreagtor();
		aggregator.resendOauth(this.compactRendering.token);
	},
	
	_updateIFrame: function() {
		var cr = this.compactRendering;
		if (!cr || !this._presentation)
			return;

		var v = dijit.getViewport();
			
		// update width first so we can measure height accurately
		
		// HoverPopup will treat a width greater than viewport - 60 as viewport - 60
		// update the effective widths so the iframe is sized according to the max
		// account for the 5 px horizontal padding around the iframe
		if (cr.smallWidth)
			this.effectiveSmallWidth = (v.w - 60 < cr.smallWidth + 5) ? v.w - 65 : cr.smallWidth;
		if (cr.largeWidth)
			this.effectiveLargeWidth = (v.w - 60 < cr.largeWidth + 5) ? v.w - 65 : cr.largeWidth;
		
		if (this.effectiveSmallWidth && this.effectiveSmallWidth + 5 > 500)
			this.maxWidth = this.effectiveSmallWidth + 5;
		if (this.effectiveLargeWidth && this.effectiveLargeWidth + 5 > 500 && (this.effectiveLargeWidth > this.effectiveSmallWidth || !this.effectiveSmallWidth))
			this.maxWidth = this.effectiveLargeWidth + 5;
		
		var w = this._isFullDisplay ? this.effectiveLargeWidth : this.effectiveSmallWidth;
		if (w) {
			this._presentation.loadingImage.style.width = this._presentation.hoverIframe.style.width = w + "px";
			if (dojo.isIE) {
				// IE needs explicit width on the divider and width 100% breaks it out the side
				// allow 4 px for margins on wrapper dix
				this._presentation.divider.style.width = (w - 4) + "px";
				this._presentation.bottomDivider.style.width = (w - 4) + "px";
			}
		}
		
		var heightOffset = this._presentation.titleBar.offsetHeight + 48;
		
		// Use dynamic heights if they're available
		var smallHeight = this.dynamicSmallHeight || cr.smallHeight;
		var largeHeight = this.dynamicLargeHeight || (cr.initialLargeHeight && this.dynamicSmallHeight) || cr.largeHeight;
		
		// HoverPopup will treat a height greater than viewport - 30 as viewport - 30
		// update the effective widths so the iframe is sized according to the max
		// account for the 45 px (+ titleBar height) vertical padding/content around the iframe using heightOffset
		if (smallHeight)
			this.effectiveSmallHeight = (v.h - 30 < smallHeight + heightOffset) ? v.h - (30 + heightOffset) : smallHeight;
		if (largeHeight)
			this.effectiveLargeHeight = (v.h - 30 < largeHeight + heightOffset) ? v.h - (30 + heightOffset) : largeHeight;
		
		if (this.effectiveSmallHeight && this.effectiveSmallHeight + heightOffset > 300)
			this.maxHeight = this.effectiveSmallHeight + heightOffset;
		if (this.effectiveLargeHeight && this.effectiveLargeHeight + heightOffset > 300 && (this.effectiveLargeHeight > this.effectiveSmallHeight || !this.effectiveSmallHeight))
			this.maxHeight = this.effectiveLargeHeight + heightOffset;
		
		var h = this._isFullDisplay ? this.effectiveLargeHeight : this.effectiveSmallHeight;
		if (h) {
			this._presentation.loadingImage.style.height = this._presentation.hoverIframe.style.height = h + "px";
		}
	},
	
	_onKeyPress: function(e) {
		if(e.type == "keypress" && e.keyCode == dojo.keys.ESCAPE)
			this.close();
	},
	
	_onBodyClick: function(e) {
		var target = e && e.target;
		if (target && !dojo.isDescendant(target, jazz.ui._getMasterPopup().domNode))
			this.close();
	},
	
	_onStopEvent: function(e) {
		if (e.type == "click")
			this._onBodyClick(e);
	},
	
	_onMessage: function(e) {
		if (!this.compactRendering || !this._presentation || e.source !== this._presentation.hoverIframe.contentWindow ||
				(this._isFullDisplay && typeof this.compactRendering.initialLargeHeight === "undefined") ||
				(!this._isFullDisplay && typeof this.compactRendering.initialSmallHeight === "undefined"))
			return;
		var data = e.data;
		if (data && data.indexOf(PREVIEW_RESIZE_MESSAGE_PREFIX) === 0) {
			var newHeight = parseInt(data.substring(PREVIEW_RESIZE_MESSAGE_PREFIX.length));
			if (this._isFullDisplay) {
				this.dynamicLargeHeight = newHeight;
			} else {
				this.dynamicSmallHeight = newHeight;
			}
			if (this._isOpen) {
				this._updateIFrame();
				this.position();
			}
		}
	},
	
	destroy: function() {
		if (this._presentation)
			this._presentation.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("net.jazz.ui._RichHoverPresentation", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ResourceLink.html"),
	messages: messages,
	
	postCreate: function() {
		if (dojo.isIE){
			dojo.addClass(this.loadingImage, "hidden");
			dojo.removeClass(this.hoverIframe, "loadingIframe");
		}
	},
	
	_contentLoaded: function(e){
		//Not checking for IE becaues there is a bug in IE where iframe contents are not shown
		//when "visibility: hidden" is removed from the iframe's CSS attribute
		if ( e.type === "load"){
			dojo.addClass(this.loadingImage, "hidden");
			dojo.removeClass(this.hoverIframe, "loadingIframe");
		}
	}
});

var processCompactRendering = function(results) {
	if (results.oauthRequired)
		return;
	
	if (results.title)
		results.title = jazz.util.html.removeScript(results.title);
	
	if (results.smallPreview || results.largePreview)
		results.showHover = true;
	
	if (results.smallPreview && results.largePreview)
		results.showExpand = true;
	
	if (results.smallWidth && (Math.ceil(results.smallWidth) == Math.floor(results.smallWidth))) 
		results.smallWidth = parseInt(results.smallWidth);
	else
		results.smallWidth = 300;
	
	if (results.smallHeight && (Math.ceil(results.smallHeight) == Math.floor(results.smallHeight))) 
		results.smallHeight = parseInt(results.smallHeight);
	else
		results.smallHeight = 200;
	
	if (results.largeWidth && (Math.ceil(results.largeWidth) == Math.floor(results.largeWidth))) 
		results.largeWidth = parseInt(results.largeWidth);
	else
		results.largeWidth = Math.max(300, results.smallWidth);
	
	if (results.largeHeight && (Math.ceil(results.largeHeight) == Math.floor(results.largeHeight))) 
		results.largeHeight = parseInt(results.largeHeight);
	else
		results.largeHeight = Math.max(300, results.smallHeight);
	
	if (results.initialSmallHeight && (Math.ceil(results.initialSmallHeight) == Math.floor(results.initialSmallHeight))) 
		results.initialSmallHeight = parseInt(results.initialSmallHeight);
	
	if (results.initialLargeHeight && (Math.ceil(results.initialLargeHeight) == Math.floor(results.initialLargeHeight))) 
		results.initialLargeHeight = parseInt(results.initialLargeHeight);
};

var parseCompactRenderingXML = function(crXml) {
	var crObj = {}, pref_width, pref_height, init_height,
		titleNodes = crXml.getElementsByTagNameNS("title", XMLNS_DC_TERMS),
		iconNodes = crXml.getElementsByTagNameNS("icon", XMLNS_JP),
		smallPreviewNodes = crXml.getElementsByTagNameNS("smallPreview", XMLNS_JP),
		largePreviewNodes = crXml.getElementsByTagNameNS("largePreview", XMLNS_JP);
	if (titleNodes.length > 0 && titleNodes[0].childNodes.length > 0) {
		crObj.title = titleNodes[0].childNodes[0].nodeValue;
	}
	if (iconNodes.length > 0) {
		crObj.icon = iconNodes[0].getAttributeNS("resource", XMLNS_RDF);
	}
	if (smallPreviewNodes.length > 0) {
		crObj.smallPreview = smallPreviewNodes[0].getAttributeNS("resource", XMLNS_RDF);
		pref_width = parseInt(smallPreviewNodes[0].getAttributeNS("preferred_width", XMLNS_JP));
		pref_height = parseInt(smallPreviewNodes[0].getAttributeNS("preferred_height", XMLNS_JP));
		init_height = parseInt(smallPreviewNodes[0].getAttributeNS("initialHeight", XMLNS_JP));
		if (pref_width) {
			crObj.smallWidth = pref_width;
		}
		if (pref_height) {
			crObj.smallHeight = pref_height;
		}
		if (init_height) {
			crObj.initialSmallHeight = init_height;
		}
	}
	if (largePreviewNodes.length > 0) {
		crObj.largePreview = largePreviewNodes[0].getAttributeNS("resource", XMLNS_RDF);
		pref_width = parseInt(largePreviewNodes[0].getAttributeNS("preferred_width", XMLNS_JP));
		pref_height = parseInt(largePreviewNodes[0].getAttributeNS("preferred_height", XMLNS_JP));
		init_height = parseInt(largePreviewNodes[0].getAttributeNS("initialHeight", XMLNS_JP));
		if (pref_width) {
			crObj.largeWidth = pref_width;
		}
		if (pref_height) {
			crObj.largeHeight = pref_height;
		}
		if (init_height) {
			crObj.initialLargeHeight = init_height;
		}
	}
	return crObj;
};

})();